module VkLongpollBot

  ##
  # Custom exceptions.
  module Exceptions
  
    ##
    # All of the error codes descriptions. Source: https://vk.com/dev/errors
    CODES = {
      1   => "Произошла неизвестная ошибка.",
      2   => "Приложение выключено.",
      3   => "Передан неизвестный метод.",
      4   => "Неверная подпись.",
      5   => "Авторизация пользователя не удалась.",
      6   => "Слишком много запросов в секунду.",
      7   => "Нет прав для выполнения этого действия.",
      8   => "Неверный запрос.",
      9   => "Слишком много однотипных действий.",
      10  => "Произошла внутренняя ошибка сервера.",
      11  => "В тестовом режиме приложение должно быть выключено или пользователь должен быть залогинен.",
      14  => "Требуется ввод кода с картинки (Captcha).",
      15  => "Доступ запрещён.",
      16  => "Требуется выполнение запросов по протоколу HTTPS, т.к. пользователь включил настройку, требующую работу через безопасное соединение.",
      17  => "Требуется валидация пользователя.",
      18  => "Страница удалена или заблокирована.",
      20  => "Данное действие запрещено для не Standalone приложений.",
      21  => "Данное действие разрешено только для Standalone и Open API приложений.",
      23  => "Метод был выключен.",
      24  => "Требуется подтверждение со стороны пользователя.",
      27  => "Ключ доступа сообщества недействителен.",
      28  => "Ключ доступа приложения недействителен.",
      29  => "Достигнут количественный лимит на вызов метода.",
      30  => "Профиль является приватным.",
      33  => "Метод ещё не реализован.",
      100 => "Один из необходимых параметров был не передан или неверен.",
      101 => "Неверный API ID приложения.",
      113 => "Неверный идентификатор пользователя.",
      150 => "Неверный timestamp.",
      200 => "Доступ к альбому запрещён.",
      201 => "Доступ к аудио запрещён.",
      203 => "Доступ к группе запрещён.",
      300 => "Альбом переполнен.",
      500 => "Действие запрещено. Вы должны включить переводы голосов в настройках приложения. ",
      600 => "Нет прав на выполнение данных операций с рекламным кабинетом.",
      603 => "Произошла ошибка при работе с рекламным кабинетом.",
      900 => "Нельзя отправлять сообщение пользователю из черного списка.",
      901 => "Пользователь запретил отправку сообщений от имени сообщества.",
      902 => "Нельзя отправлять сообщения этому пользователю в связи с настройками приватности.",
      911 => "Формат клавиатуры некорректен.",
      912 => "Это функция чат-бота, измените данный статус в настройках.",
      913 => "Слишком много пересланных сообщений.",
      914 => "Сообщение слишком длинное.",
      917 => "У вас нет доступа к этому чату.",
      921 => "Невозможно переслать выбранные сообщения.",
      936 => "Контакт не найден.",
      940 => "Слишком много постов в сообщении."
    }
    
    ##
    # Something wrong with response from vk.com.
    class ResponseError < RuntimeError
    end
    
    ##
    # API error. Must have some code and description.
    class APIError < ResponseError

      attr_reader :error
      
      ##
      # Create new instance.
      #
      # @param error [Hash] response hash which contains "error" key.
      def initialize(error)
        @error = error["error"]
        super("#{code}: #{included_message}")
      end

      ##
      # @return [Integer]
      def code
        @error["error_code"]
      end

      ##
      # @return [String] attached error description.
      def included_message
        @error["error_msg"]
      end
      
      ##
      # @return [String] error description from API documentation.
      def description
        CODES[code]
      end
    
    end
    
    ##
    # Something wrong with longpoll response.
    class LongpollError < ResponseError
    end
  
  end

end
